/** 
   Copyright (C) 2008 Free Software Foundation, Inc.
   
   Written by:  Richard Frith-Macdonald <rfm@gnu.org>
   Date:	January 2008
   
   This file is part of the WebServices Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   $Date: 2007-09-14 13:54:55 +0100 (Fri, 14 Sep 2007) $ $Revision: 25485 $
   */ 

#ifndef	INCLUDED_WEBSERVICES_H
#define	INCLUDED_WEBSERVICES_H

#import <WebServices/GWSBinding.h>
#import <WebServices/GWSCoder.h>
#import <WebServices/GWSConstants.h>
#import <WebServices/GWSDocument.h>
#import <WebServices/GWSElement.h>
#import <WebServices/GWSExtensibility.h>
#import <WebServices/GWSHash.h>
#import <WebServices/GWSMessage.h>
#import <WebServices/GWSPort.h>
#import <WebServices/GWSPortType.h>
#import <WebServices/GWSService.h>
#import <WebServices/GWSType.h>

#import <WebServices/WSSUsernameToken.h>

#endif

